import json
import os

from django.test import TestCase

from scan_models.main import scan_model


class TestScanModel(TestCase):
    def test_happy(self):
        path = "./output.json"
        scan_model("tests.TestModel", path)

        with open(os.path.abspath(path), "r") as file:
            data = json.load(file)
            response = {
                "name": {"validator": {"max": 30}},
                "email": {"validator": {"required": True, "email": True, "max": 254}},
                "maxAmount": {"validator": {"required": True}, "attributes": {"type": "number"}},
                "minAmount": {"validator": {"required": True}, "attributes": {"type": "number"}},
            }

            self.assertEqual(data, response)



from scan_models.validators.base_validator import BaseValidator


class VeeValidate(BaseValidator):
    def set_required(self, validator: dict, required: bool):
        validator["required"] = required

    def set_max_length(self, validator: dict, max_length: int):
        validator["max"] = max_length

    def set_max_value(self, validator: dict, max_value: float):
        validator["max_value"] = max_value

    def set_min_value(self, validator: dict, min_value: float):
        validator["min_value"] = min_value

    def set_is_email(self, validator: dict):
        validator["email"] = True



class BaseValidator:
    def set_required(self, validator: dict, required: bool):
        pass

    def set_max_length(self, validator: dict, max_length: int):
        pass

    def set_max_value(self, validator: dict, max_value: float):
        pass

    def set_min_value(self, validator: dict, min_value: float):
        pass

    def set_is_email(self, validator: dict):
        pass



from django.conf import settings

SETTING_NAME = "SCAN_MODELS"

# todo: add custom validator to readme
DEFAULT_SETTINGS = {"mapping": {}, "validator": "scan_models.validators.VeeValidate"}


def get_setting(name: str):
    """
    Gets the setting for scan models

    Args:
        name: Name of the setting

    Returns: Value fo the setting
    """

    setting = getattr(settings, SETTING_NAME) if hasattr(settings, SETTING_NAME) else {}

    return setting[name] if name in setting else DEFAULT_SETTINGS[name]



import json
import os

from django.apps import apps

from scan_models.parser import FieldParser


def scan_model(model_name: str, output: str):
    """
    Scan the model and output it to a file

    Args:
        model_name (str): The name of the model. Preferably with the app. Example: companies.Company
        output (str): Output file. Advised to use .json on the end
    """

    fields = apps.get_model(model_name)._meta.fields

    validator = {}

    for field in fields:
        validator_field = FieldParser(field).parse()

        if validator_field:
            validator[_snake_to_camel(field.name)] = validator_field

    with open(os.path.abspath(output), "w") as outfile:
        json.dump(validator, outfile, indent=2)


def _snake_to_camel(value: str):
    """
    Snake casing to camel

    Args:
        value: The string to format
    """
    components = value.split("_")
    return components[0] + "".join(x.title() for x in components[1:])



from django.core.management import BaseCommand

from scan_models.main import scan_model
from scan_models.settings import get_setting


class Command(BaseCommand):
    help = "Creates the hour registration for the last week"

    def add_arguments(self, parser):
        # todo: Add this to readme
        parser.add_argument(
            "--model", help="If you only want to specify one model",
        )

    def handle(self, *args, **options):
        data = get_setting("mapping")
        option_model = options.get("model", None)

        if option_model:
            scan_model(option_model, data[option_model])
        else:
            for model, output in data.items():
                scan_model(model, output)



from typing import Type

from scan_models.settings import get_setting
from scan_models.validators.base_validator import BaseValidator
from pydoc import locate

# todo: Add validator list to readme
class ValidatorFactory:
    @staticmethod
    def get_validator() -> Type[BaseValidator]:
        return locate(get_setting("validator"))



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db.models import fields

from scan_models.validators.base_validator import BaseValidator
from scan_models.validators.factory import ValidatorFactory


class FieldParser:
    field: fields.Field
    validator = {"name": " sda"}
    validator_class: BaseValidator

    def __init__(self, field: fields.Field):
        self.validator_class = ValidatorFactory.get_validator()()
        self.field = field

        super().__init__()

    def parse(self):
        parsed = {}

        self.validator = {}
        self._parse_validator()
        if self.validator:
            parsed["validator"] = self.validator

        attributes = self._parse_attributes()
        if attributes:
            parsed["attributes"] = attributes

        return parsed

    def _parse_attributes(self):
        attributes = {}
        if type(self.field) == fields.IntegerField:
            attributes["type"] = "number"

        return attributes

    def _parse_validator(self):
        self._calculate_required()
        self._calculate_email()
        self._calculate_max_length()
        # self._calculate_max_min_value()

    def _calculate_required(self):
        required = self.field.default == fields.NOT_PROVIDED and not self.field.blank and not self.field.null

        if required:
            self.validator_class.set_required(self.validator, required)

    def _calculate_email(self):
        if type(self.field) == fields.EmailField:
            self.validator_class.set_is_email(self.validator)

    def _calculate_max_length(self):
        if self.field.max_length:
            self.validator_class.set_max_length(self.validator, self.field.max_length)

    def _calculate_max_min_value(self):
        if type(self.field) != fields.IntegerField:
            return

        max_validator = [validator for validator in self.field.validators if type(validator) == MaxValueValidator][
            0
        ].limit_value
        min_validator = [validator for validator in self.field.validators if type(validator) == MinValueValidator][
            0
        ].limit_value

        internal_type = self.field.get_internal_type()
        min_value, max_value = fields.connection.ops.integer_field_range(internal_type)

        if max_validator != max_value:
            self.validator_class.set_max_value(self.validator, max_validator)

        if min_validator != min_value:
            self.validator_class.set_min_value(self.validator, min_validator)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['scan_models',
 'scan_models.management',
 'scan_models.management.commands',
 'scan_models.migrations',
 'scan_models.validators']

package_data = \
{'': ['*']}

install_requires = \
['django>=3.0.4,<4.0.0', 'mysqlclient>=1.4.6,<2.0.0']

setup_kwargs = {
    'name': 'django-scan-models',
    'version': '0.1.3',
    'description': 'Django scan models: Parse django models to frontend validation',
    'long_description': None,
    'author': 'Jessie Liauw A Fong',
    'author_email': 'jessielaff@live.nl',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
